// Copyright (c) HashiCorp, Inc.
// SPDX-License-Identifier: MPL-2.0

package roles

import (
	"context"
	"fmt"
	"net/http"
	"reflect"
	"testing"

	"github.com/go-openapi/runtime/client"
	"github.com/hashicorp/hcp-sdk-go/clients/cloud-resource-manager/stable/2019-12-10/client/organization_service"
	"github.com/hashicorp/hcp-sdk-go/clients/cloud-resource-manager/stable/2019-12-10/models"
	cloud "github.com/hashicorp/hcp-sdk-go/clients/cloud-shared/v1/models"
	"github.com/stretchr/testify/mock"
	"github.com/stretchr/testify/require"

	mock_organization_service "github.com/hashicorp/hcp/internal/pkg/api/mocks/github.com/hashicorp/hcp-sdk-go/clients/cloud-resource-manager/stable/2019-12-10/client/organization_service"
	"github.com/hashicorp/hcp/internal/pkg/cmd"
	"github.com/hashicorp/hcp/internal/pkg/format"
	"github.com/hashicorp/hcp/internal/pkg/iostreams"
	"github.com/hashicorp/hcp/internal/pkg/profile"
)

func TestNewCmdList(t *testing.T) {
	t.Parallel()

	cases := []struct {
		Name    string
		Args    []string
		Profile func(t *testing.T) *profile.Profile
		Error   string
	}{
		{
			Name:    "No Org",
			Profile: profile.TestProfile,
			Args:    []string{},
			Error:   "Organization ID must be configured",
		},
		{
			Name: "Too many args",
			Profile: func(t *testing.T) *profile.Profile {
				return profile.TestProfile(t).SetOrgID("123")
			},
			Args:  []string{"foo", "bar"},
			Error: "no arguments allowed, but received 2",
		},
		{
			Name: "Good",
			Profile: func(t *testing.T) *profile.Profile {
				return profile.TestProfile(t).SetOrgID("123")
			},
			Args: []string{},
		},
	}

	for _, c := range cases {
		c := c
		t.Run(c.Name, func(t *testing.T) {
			t.Parallel()
			r := require.New(t)

			// Create a context.
			io := iostreams.Test()
			ctx := &cmd.Context{
				IO:          io,
				Profile:     c.Profile(t),
				Output:      format.New(io),
				HCP:         &client.Runtime{},
				ShutdownCtx: context.Background(),
			}

			cmd := NewCmdList(ctx, func(o *ListOpts) error {
				return nil
			})
			cmd.SetIO(io)

			code := cmd.Run(c.Args)
			if c.Error != "" {
				r.NotZero(code)
				r.Contains(io.Error.String(), c.Error)
				return
			}

			r.Zero(code, io.Error.String())
		})
	}
}

func TestListRun(t *testing.T) {
	t.Parallel()

	cases := []struct {
		Name    string
		Resp    [][]*models.HashicorpCloudResourcemanagerRole
		RespErr bool
		Error   string
	}{
		{
			Name:    "Server error",
			RespErr: true,
			Error:   "failed to list organization roles: [GET /resource-manager/2019-12-10/organizations/{id}/roles][403]",
		},
		{
			Name: "Good no pagination",
			Resp: [][]*models.HashicorpCloudResourcemanagerRole{
				{
					{
						ID:          "roles/admin",
						Title:       "Admin",
						Description: "Role Admin description",
					},
				},
			},
		},
		{
			Name: "Good pagination",
			Resp: [][]*models.HashicorpCloudResourcemanagerRole{
				{
					{
						ID:          "roles/admin",
						Title:       "Admin",
						Description: "Role Admin description",
					},
					{
						ID:          "roles/contributor",
						Title:       "Contributor",
						Description: "describing contributor",
					},
				},
				{
					{
						ID:          "roles/viewer",
						Title:       "Admin",
						Description: "describing viewer",
					},
					{
						ID:          "roles/vault-secrets.Admin",
						Title:       "Vault Secrets Admin",
						Description: "manage all the secrets",
					},
				},
			},
		},
	}

	for _, c := range cases {
		c := c
		t.Run(c.Name, func(t *testing.T) {
			t.Parallel()
			r := require.New(t)

			io := iostreams.Test()
			org := mock_organization_service.NewMockClientService(t)
			opts := &ListOpts{
				Ctx:     context.Background(),
				Profile: profile.TestProfile(t).SetOrgID("123"),
				IO:      io,
				Output:  format.New(io),
				Client:  org,
			}

			e := org.EXPECT()
			for i := 0; i < len(c.Resp) || c.RespErr; i++ {
				i := i
				call := e.OrganizationServiceListRoles(mock.MatchedBy(func(req *organization_service.OrganizationServiceListRolesParams) bool {
					// Expect an org
					if req.ID != "123" {
						return false
					}

					// No initial pagination
					if i == 0 && req.PaginationNextPageToken != nil {
						return false
					} else if i >= 1 && *req.PaginationNextPageToken != fmt.Sprintf("next-page-%d", i) {
						// Expect a page token
						return false
					}

					return true
				}), nil)

				if c.RespErr {
					call.Return(nil, organization_service.NewOrganizationServiceListRolesDefault(http.StatusForbidden))
					break
				} else {
					ok := organization_service.NewOrganizationServiceListRolesOK()
					ok.Payload = &models.HashicorpCloudResourcemanagerOrganizationListRolesResponse{
						Roles: c.Resp[i],
					}

					if i < len(c.Resp)-1 {
						ok.Payload.Pagination = &cloud.HashicorpCloudCommonPaginationResponse{
							NextPageToken: fmt.Sprintf("next-page-%d", i+1),
						}
					}

					call.Return(ok, nil)
				}
			}

			// Run the command
			err := listRun(opts)
			if c.Error != "" {
				r.ErrorContains(err, c.Error)
				return
			}

			r.NoError(err)

			// Check we outputted the project
			for _, page := range c.Resp {
				for _, p := range page {
					r.Contains(io.Output.String(), p.ID)
					r.Contains(io.Output.String(), p.Title)
					r.Contains(io.Output.String(), p.Description)
				}
			}
		})
	}
}

func Test_sortRoles(t *testing.T) {
	t.Parallel()

	tests := []struct {
		name       string
		inputRoles []*models.HashicorpCloudResourcemanagerRole
		want       []*models.HashicorpCloudResourcemanagerRole
	}{
		{
			name: "simple",
			inputRoles: []*models.HashicorpCloudResourcemanagerRole{
				{ID: "roles/zookeeper"},
				{ID: "roles/cat-wrangler"},
				{ID: "roles/admin"},
				{ID: "roles/!not-valid"},
			},
			want: []*models.HashicorpCloudResourcemanagerRole{
				{ID: "roles/!not-valid"},
				{ID: "roles/admin"},
				{ID: "roles/cat-wrangler"},
				{ID: "roles/zookeeper"},
			},
		},
		{
			name: "grouped",
			inputRoles: []*models.HashicorpCloudResourcemanagerRole{
				{ID: "roles/iam.zookeeper"},
				{ID: "roles/secrets.secret-reader"},
				{ID: "roles/viewer"},
				{ID: "roles/contributor"},
				{ID: "roles/the.future.isUnknown"},
				{ID: "roles/iam.admin"},
				{ID: "roles/admin"},
				{ID: "roles/secrets.app-manager"},
				{ID: "roles/the.feature.isSafe"},
			},
			want: []*models.HashicorpCloudResourcemanagerRole{
				{ID: "roles/admin"},
				{ID: "roles/contributor"},
				{ID: "roles/viewer"},
				{ID: "roles/iam.admin"},
				{ID: "roles/iam.zookeeper"},
				{ID: "roles/secrets.app-manager"},
				{ID: "roles/secrets.secret-reader"},
				{ID: "roles/the.feature.isSafe"},
				{ID: "roles/the.future.isUnknown"},
			},
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			t.Parallel()
			if got := sortRolesByID(tt.inputRoles); !reflect.DeepEqual(got, tt.want) {
				t.Errorf("Sorting failed.\nExpected:\n")
				for _, role := range tt.want {
					t.Errorf("%s\n", role.ID)
				}
				t.Errorf("Got:\n")
				for _, role := range got {
					t.Errorf("%s\n", role.ID)
				}
			}
		})
	}
}
