// Copyright (c) HashiCorp, Inc.
// SPDX-License-Identifier: MPL-2.0

package addons

import (
	"context"
	"testing"

	"github.com/go-openapi/runtime/client"
	"github.com/hashicorp/hcp/internal/pkg/cmd"
	"github.com/hashicorp/hcp/internal/pkg/format"
	"github.com/hashicorp/hcp/internal/pkg/iostreams"
	"github.com/hashicorp/hcp/internal/pkg/profile"
	"github.com/stretchr/testify/require"
)

func TestNewCmdCreate(t *testing.T) {
	t.Parallel()

	cases := []struct {
		Name    string
		Args    []string
		Profile func(t *testing.T) *profile.Profile
		Error   string
		Expect  *AddOnOpts
	}{
		{
			Name:    "No Org",
			Profile: profile.TestProfile,
			Args:    []string{},
			Error:   "Organization ID must be configured",
		},
		{
			Name: "no args",
			Profile: func(t *testing.T) *profile.Profile {
				return profile.TestProfile(t).SetOrgID("123")
			},
			Args:  []string{},
			Error: "accepts 1 arg(s), received 0",
		},
		{
			Name: "happy",
			Profile: func(t *testing.T) *profile.Profile {
				return profile.TestProfile(t).SetOrgID("123")
			},
			Args: []string{
				"-n=cli-test",
				"--app=testApp",
				"--add-on-definition-name=testAddOnDefinition",
				"--var-file", "vars.hcl",
				"--var", "key=value",
				"--var", "key2=value2",
			},
			Expect: &AddOnOpts{
				Name:                "cli-test",
				ApplicationName:     "testApp",
				AddOnDefinitionName: "testAddOnDefinition",
				Variables: map[string]string{
					"key":  "value",
					"key2": "value2",
				},
				VariablesFile: "vars.hcl",
			},
		},
	}

	for _, c := range cases {
		c := c
		t.Run(c.Name, func(t *testing.T) {
			t.Parallel()

			r := require.New(t)

			io := iostreams.Test()
			ctx := &cmd.Context{
				IO:          io,
				Profile:     c.Profile(t),
				ShutdownCtx: context.Background(),
				HCP:         &client.Runtime{},
				Output:      format.New(io),
			}

			var addOnOpts AddOnOpts
			addOnOpts.testFunc = func(c *cmd.Command, args []string) error {
				return nil
			}
			cmd := NewCmdCreate(ctx, &addOnOpts)
			cmd.SetIO(io)

			cmd.Run(c.Args)

			if c.Expect != nil {
				r.Equal(c.Expect.Name, addOnOpts.Name)
				r.Equal(c.Expect.ApplicationName, addOnOpts.ApplicationName)
				r.Equal(c.Expect.AddOnDefinitionName, addOnOpts.AddOnDefinitionName)
				r.Equal(c.Expect.Variables, addOnOpts.Variables)
				r.Equal(c.Expect.VariablesFile, addOnOpts.VariablesFile)
			}
		})
	}
}
